import 'package:flutter/material.dart';
import 'package:mobile_ui/mobile_ui.dart';

enum BaseTextType {
  h1,
  h2,
  h3,
  h4,
  h5,
  h6,
  subtitle1,
  subtitle2,
  body1,
  body2,
  caption,
  overline
}

class Label extends StatelessWidget {
  final String text;
  final BaseTextType? type;
  final Color? color;
  final TextAlign textAlign;
  final int? maxLines;
  final bool softWrap;
  final int? fontWeightDelta;
  final TextOverflow? overflow;
  final FontStyle? fontStyle;
  final FontWeight? fontWeight;
  final double? fontSize;

  Label(this.text,
      {this.type,
      this.color,
      this.textAlign = TextAlign.left,
      this.maxLines,
      this.softWrap = true,
      this.overflow,
      this.fontWeightDelta,
      this.fontWeight,
      this.fontStyle,
      this.fontSize});

  @override
  Widget build(BuildContext context) {
    var textTheme = Theme.of(context).textTheme;
    var textStyle;
    switch (type) {
      case BaseTextType.h1:
        textStyle = textTheme.headline1;
        break;
      case BaseTextType.h2:
        textStyle = textTheme.headline2;
        break;
      case BaseTextType.h3:
        textStyle = textTheme.headline3;
        break;
      case BaseTextType.h4:
        textStyle = textTheme.headline4;
        break;
      case BaseTextType.h5:
        textStyle = textTheme.headline5;
        break;
      case BaseTextType.h6:
        textStyle = textTheme.headline6;
        break;
      case BaseTextType.subtitle1:
        textStyle = textTheme.subtitle1;
        break;
      case BaseTextType.subtitle2:
        textStyle = textTheme.subtitle2;
        break;
      case BaseTextType.body1:
        textStyle = textTheme.bodyText2;
        break;
      case BaseTextType.body2:
        textStyle = textTheme.bodyText1;
        break;
      case BaseTextType.caption:
        textStyle = textTheme.caption;
        break;
      case BaseTextType.overline:
        textStyle = textTheme.overline;
        break;
      default:
        textStyle = DefaultTextStyle.of(context).style;
        break;
    }
    if (color != null) {
      textStyle = textStyle.copyWith(color: color);
    } else {
      final defaultLabelColor = MobileUIContainer.of(context)?.labelColor;
      if (defaultLabelColor != null) {
        textStyle = textStyle.copyWith(color: defaultLabelColor);
      }
    }
    if (fontWeight != null) {
      textStyle = textStyle.copyWith(fontWeight: fontWeight);
    }
    if (fontWeightDelta != null) {
      textStyle = textStyle.apply(fontWeightDelta: fontWeightDelta);
    }
    if (fontStyle != null) {
      textStyle = textStyle.copyWith(fontStyle: fontStyle);
    }
    if (fontSize != null) {
      textStyle = textStyle.copyWith(fontSize: fontSize);
    }
    return Text(
      text,
      style: textStyle,
      textAlign: textAlign,
      maxLines: maxLines,
      overflow: TextOverflow.ellipsis,
      softWrap: softWrap,
    );
  }
}
