import 'package:flutter/material.dart';

import '../../mobile_ui.dart';

class RadioListFormInput<T> extends FormInputBase<T> {
  RadioListFormInput(
      {Key? key,
      String? labelText,
      T? initialValue,
      ValueController<T>? controller,
      FocusNode? focusNode,
      bool enabled = true,
      bool isRequired = false,
      Map<String, T>? itemSource,
      FormFieldSetter<T>? onSaved,
      ValueValidator<T?>? validator,
      bool? isDense,
      ValueChanged<T?>? onChanged,
      double spacingFlactor = 1,
      AutovalidateMode? autovalidateMode,
      TextAlign labelAlign = TextAlign.left,
      Axis direction = Axis.horizontal,
      FieldOrientation orientation = FieldOrientation.vertical,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      ControlPosition controlPosition = ControlPosition.trailing,
      double? labelWidth,
      int errorMaxLines = 1,
      FormInputLabelStyle? labelStyle})
      : super(
          key: key,
          initialValue: initialValue,
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          isDense: isDense,
          autovalidateMode: autovalidateMode,
          labelWidth: labelWidth,
          spacingFlactor: spacingFlactor,
          orientation: orientation,
          controlPosition: controlPosition,
          labelText: labelText,
          labelAlign: labelAlign,
          valueController: controller,
          isRequired: isRequired,
          contentPadding: contentPadding,
          onChanged: onChanged,
          errorMaxLines: errorMaxLines,
          margin: margin,
          labelStyle: labelStyle,
          backgroundStyle: FormInputBackgroundStyle.none,
          builder: (field) {
            FormInputBase<T> widget = field.widget as FormInputBase<T>;
            FormInputStateBase<T> state = field as FormInputStateBase<T>;
            var children = <Widget>[];
            itemSource!.forEach((key, value) {
              children.add(RadioItem<T>(
                labelText: key,
                value: value,
                enabled: enabled,
                onChanged: (v) {
                  state.changeValue(v);
                },
                groupValue: state.effectiveValueController!.value,
                controlPosition: controlPosition,
              ));
            });
            return widget.buildField(
                field,
                Wrap(
                  spacing: uiSpacing(spacingFlactor),
                  direction: direction,
                  children: children,
                ));
          },
        );
}

class RadioItem<T> extends StatelessWidget {
  final String? labelText;
  final T? value;
  final T? groupValue;
  final FocusNode? focusNode;

  final bool enabled;
  final ValueChanged<T?>? onChanged;
  final ControlPosition controlPosition;

  RadioItem(
      {Key? key,
      this.labelText,
      this.value,
      this.enabled = true,
      this.focusNode,
      this.onChanged,
      this.groupValue,
      this.controlPosition = ControlPosition.leading})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final label = Text(labelText ?? '',
        style: enabled
            ? null
            : theme.textTheme.subtitle1!.copyWith(color: theme.disabledColor));
    var radio = Radio(
        focusNode: focusNode,
        groupValue: groupValue,
        onChanged: enabled == true
            ? (dynamic value) {
                if (onChanged != null) onChanged!(value);
              }
            : null,
        value: value,
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap);
    var children = <Widget>[label];
    if (controlPosition == ControlPosition.leading) {
      children.insert(0, radio);
    } else {
      children.add(radio);
    }
    return InkWell(
        onTap: () {
          if (enabled == true) {
            if (onChanged != null) onChanged!(value);
          }
        },
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: children,
        ));
  }
}
